import { echo } from "@atums/echo";
import { redis } from "bun";
import type { ServerWebSocket } from "bun";

class WebSocketHandler {
	private chatRooms: Map<string, Set<ServerWebSocket>> = new Map();
	private userSockets: Map<ServerWebSocket, UserInfo> = new Map();

	public handleMessage(ws: ServerWebSocket, message: string): void {
		try {
			const parsedMessage: WebSocketMessage = JSON.parse(message);

			switch (parsedMessage.type) {
				case "join_chat":
					this.handleJoinChat(ws, parsedMessage);
					break;
				case "send_message":
					this.handleSendMessage(ws, parsedMessage);
					break;
				case "leave_chat":
					this.handleLeaveChat(ws, parsedMessage);
					break;
				case "load_history":
					this.handleLoadHistory(ws, parsedMessage);
					break;
				default:
					this.sendError(ws, "Unknown message type");
			}
		} catch {
			this.sendError(ws, "Invalid message format");
		}
	}

	public handleOpen(ws: ServerWebSocket): void {
		this.userSockets.set(ws, {});

		try {
			ws.send(
				JSON.stringify({
					type: "connected",
					data: { message: "Connected to chat server" },
				}),
			);
		} catch {}
	}

	public handleClose(ws: ServerWebSocket): void {
		const userInfo = this.userSockets.get(ws);
		if (userInfo?.streamId) {
			this.removeFromChatRoom(ws, userInfo.streamId);
			this.broadcastUserLeft(
				userInfo.streamId,
				userInfo.username || "Anonymous",
			);
		}

		this.userSockets.delete(ws);
	}

	private async handleLoadHistory(
		ws: ServerWebSocket,
		message: WebSocketMessage,
	): Promise<void> {
		const { streamId } = message;

		if (!streamId) {
			this.sendError(ws, "StreamId is required");
			return;
		}

		try {
			const chatHistory = await this.getChatHistory(streamId);
			this.sendResponse(ws, {
				type: "chat_history",
				data: { messages: chatHistory, streamId },
			});
		} catch {
			this.sendError(ws, "Failed to load chat history");
		}
	}

	private async handleJoinChat(
		ws: ServerWebSocket,
		message: WebSocketMessage,
	): Promise<void> {
		const { streamId, username } = message;

		if (!streamId || !username) {
			this.sendError(ws, "StreamId and username are required");
			return;
		}

		try {
			const userInfo = this.userSockets.get(ws);
			if (userInfo?.streamId && userInfo.streamId !== streamId) {
				this.removeFromChatRoom(ws, userInfo.streamId);
			}

			this.addToChatRoom(ws, streamId);
			this.userSockets.set(ws, { streamId, username });

			const chatHistory = await this.getChatHistory(streamId);
			this.sendResponse(ws, {
				type: "chat_history",
				data: { messages: chatHistory, streamId },
			});

			this.broadcastUserJoined(streamId, username, ws);
		} catch {
			this.sendError(ws, "Failed to join chat");
		}
	}

	private async handleSendMessage(
		ws: ServerWebSocket,
		message: WebSocketMessage,
	): Promise<void> {
		const userInfo = this.userSockets.get(ws);
		const { message: messageText } = message;

		if (!userInfo?.streamId || !userInfo?.username) {
			this.sendError(ws, "You must join a chat room first");
			return;
		}

		if (!messageText || messageText.trim().length === 0) {
			this.sendError(ws, "Message cannot be empty");
			return;
		}

		if (messageText.length > 500) {
			this.sendError(ws, "Message too long (max 500 characters)");
			return;
		}

		try {
			const chatMessage: ChatMessage = {
				id: crypto.randomUUID(),
				username: userInfo.username,
				message: messageText.trim(),
				timestamp: Date.now(),
				streamId: userInfo.streamId,
			};

			await this.saveChatMessage(userInfo.streamId, chatMessage);

			this.broadcastToChatRoom(userInfo.streamId, {
				type: "chat_message",
				data: chatMessage,
			});
		} catch {
			this.sendError(ws, "Failed to send message");
		}
	}

	private handleLeaveChat(
		ws: ServerWebSocket,
		message: WebSocketMessage,
	): void {
		const userInfo = this.userSockets.get(ws);
		const { streamId } = message;

		const targetStreamId = streamId || userInfo?.streamId;

		if (targetStreamId) {
			this.removeFromChatRoom(ws, targetStreamId);
			this.broadcastUserLeft(targetStreamId, userInfo?.username || "Anonymous");

			this.userSockets.set(ws, { username: userInfo?.username });
		}
	}

	private addToChatRoom(ws: ServerWebSocket, streamId: string): void {
		if (!ws || !streamId) {
			return;
		}

		if (!this.chatRooms.has(streamId)) {
			this.chatRooms.set(streamId, new Set());
		}

		const room = this.chatRooms.get(streamId);

		if (room) {
			room.add(ws);
		}
	}

	private removeFromChatRoom(ws: ServerWebSocket, streamId: string): void {
		const room = this.chatRooms.get(streamId);
		if (room) {
			room.delete(ws);
			if (room.size === 0) {
				this.chatRooms.delete(streamId);
			}
		}
	}

	private broadcastToChatRoom(
		streamId: string,
		response: WebSocketResponse,
	): void {
		const room = this.chatRooms.get(streamId);
		if (room) {
			const message = JSON.stringify(response);
			for (const ws of room) {
				try {
					ws.send(message);
				} catch {
					room.delete(ws);
				}
			}
		}
	}

	private broadcastUserJoined(
		streamId: string,
		username: string,
		excludeWs: ServerWebSocket,
	): void {
		const room = this.chatRooms.get(streamId);
		if (room) {
			const message = JSON.stringify({
				type: "user_joined",
				data: { username, streamId },
			});

			for (const ws of room) {
				if (ws !== excludeWs) {
					try {
						ws.send(message);
					} catch {
						room.delete(ws);
					}
				}
			}
		}
	}

	private broadcastUserLeft(streamId: string, username: string): void {
		this.broadcastToChatRoom(streamId, {
			type: "user_left",
			data: { username, streamId },
		});
	}

	private sendResponse(ws: ServerWebSocket, response: WebSocketResponse): void {
		try {
			ws.send(JSON.stringify(response));
		} catch {
			this.sendError(ws, "Failed to send response");
		}
	}

	private sendError(ws: ServerWebSocket, error: string): void {
		this.sendResponse(ws, { type: "error", error });
	}

	private async getChatHistory(streamId: string): Promise<ChatMessage[]> {
		try {
			const chatKey = `chat:${streamId}`;
			const messagesData = await redis.get(chatKey);

			if (messagesData) {
				const messages = JSON.parse(messagesData);
				if (Array.isArray(messages)) {
					return messages.sort((a, b) => a.timestamp - b.timestamp);
				}
			}
			return [];
		} catch {
			return [];
		}
	}

	private async saveChatMessage(
		streamId: string,
		message: ChatMessage,
	): Promise<void> {
		try {
			const chatKey = `chat:${streamId}`;

			const existingData = await redis.get(chatKey);
			let messages: ChatMessage[] = [];

			if (existingData) {
				try {
					messages = JSON.parse(existingData);
					if (!Array.isArray(messages)) {
						messages = [];
					}
				} catch {
					messages = [];
				}
			}

			messages.push(message);

			if (messages.length > 100) {
				messages = messages.slice(-100);
			}

			await redis.set(chatKey, JSON.stringify(messages));
			await redis.expire(chatKey, 86400);

			const activeRoomsData = await redis.get("chat:active_rooms");
			let activeRooms: string[] = [];

			if (activeRoomsData) {
				try {
					activeRooms = JSON.parse(activeRoomsData);
					if (!Array.isArray(activeRooms)) {
						activeRooms = [];
					}
				} catch {
					activeRooms = [];
				}
			}

			if (!activeRooms.includes(streamId)) {
				activeRooms.push(streamId);
				await redis.set("chat:active_rooms", JSON.stringify(activeRooms));
				await redis.expire("chat:active_rooms", 86400);
			}
		} catch (error) {
			echo.error({ message: "Error saving chat message", error });
			throw error;
		}
	}

	public getChatRoomInfo(): { [streamId: string]: number } {
		const info: { [streamId: string]: number } = {};
		this.chatRooms.forEach((connections, streamId) => {
			info[streamId] = connections.size;
		});
		return info;
	}
}

const webSocketHandler: WebSocketHandler = new WebSocketHandler();
export { webSocketHandler, WebSocketHandler };
